"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.print = void 0;
const filter_1 = require("./filter");
const parser_1 = require("./parser");
const reader_1 = require("./reader");
/**
 * The main entrance to dealt with code processing.
 *
 * @param code raw text input
 * @param directives the collection of user-defined directives
 * @param params the collection of user-defined params
 * @param verbose should output omitted lines
 * @returns the processed code
 */
function print(code, directives, params, verbose) {
    let result = "";
    let lastCommentOpen = "";
    let lastCommentClose = "";
    let filter = new filter_1.PlainTextFilter();
    for (const block of (0, reader_1.reader)(code)) {
        const blockInfo = (0, parser_1.parse)(block);
        const { preserve, filter: nextFilter } = filter.next(blockInfo, directives, params);
        filter = nextFilter;
        const { c_open, c_close } = block;
        lastCommentOpen = c_open || lastCommentOpen;
        lastCommentClose = c_close || lastCommentClose;
        const { raw, isComment, isDirective } = blockInfo;
        let segment = "";
        if (preserve) {
            segment = raw;
        }
        else if (verbose) {
            if (isComment || isDirective) {
                segment = raw;
            }
            else {
                segment = verbosePrint(lastCommentOpen, lastCommentClose, raw);
            }
        }
        result += segment;
    }
    return result;
}
exports.print = print;
function verbosePrint(open, close, block) {
    return `${open} ${block.trim()}${close.trim()}\n`;
}
